# Redux

We are using **Redux** for our application state management. it maintains the state of the entire application in a single immutable state tree (object), which can't be changed directly. When something changes, a new object is created (using actions and reducers). For more information about Redux, visit The [official documentation](https://redux.js.org/).

## Store

The following are the existing store entities used by our application:

- `Profile`: Preserves the state of tutorials, user profile, public profile, introduction, and notifications (see initial state below).
- `Global`: Preserves the global state, which includes `isPublic` `toast` and `error`
- `Appointment`: Preserves the state of tutorials and notifications.
- `UploadDocument`: Manages the state of uploaded documents.

### Initial states

#### Profile

```js
{
  tutorial: { // Tells if a tutorial has already been seen
    contacts: true,
    documents: true,
    actions: {
      first: true, // 'first' represents the actions component with no appointments yet
      second: true, // 'second' represents the actions component with an existing appointment
    },
  },
  fullname: '',
  introduction: true, // If 'true', the user has already used the app and seen the introduction
  notification: false,
  lastConnectionDate: '', // ISO string
  hasPhone: false,
  userID: '',
  publicProfile: {
    fullname: '',
    endDate: '', // Timestamp
    isActive: false,
  },
}
```

#### Global

```js
{
  isPublic: false,
  toast: null, // Manages global messages toasts 
  error: false, // Manages global errors
}
```

#### Appointment

```js
{
  id: '',
  appointmentDate: '', // ISO string
  note: {
    id: '',
    content: '',
    createdAt: '',  // ISO string
  },
  objectives: [],
}
```

#### UploadDocument

```js
{
  file: null,
  qualification: null,
}
```

For each reducer file, a test file is associated testing the reducer.

All the actions are tested in a separate file with the same actions file name.

## Used packages

- **react-redux**: Official React bindings for [Redux](https://github.com/reduxjs/redux).
- **react-redux**: [Thunk middleware](https://github.com/reduxjs/redux-thunk) for Redux.
- **redux-devtools-extension**: For application's state changes debugging.
- **@types/react-redux**: This package contains type definitions for react-redux.
