# Use services in local

## Configure the cozy-stack on local

In order to be able to use your service, follow these steps:

1/ clone [cozy-doctypes](https://github.com/cozy/cozy-doctypes) repository into your system.
Then, push it into your root project.

2/ After that, add your own remote doctype into cozy-doctype folder:

    - create a new folder ex: org.mps.share
    - push into this folder request file: request

You must declare this doctype into your manifest.webapp file like that :

```json
"share": {
  "description": "Required for searching on back-endTS",
  "type": "org.mps.share",
  "verbs": ["GET", "POST"]
},
```

3/ In cozy-app-dev-with-app.sh, you need to add another option when you serve the cozy-stack:

```sh
cozy-stack serve --allow-root (...) --doctypes /path/to/the/doctype
```

=> in this project :

```sh
cozy-stack serve (...) --doctypes /data/cozy-doctypes
```

4/ You need to add another option when you install apps in the cozy-stack (go to: docker\cozy-app-dev-with-app.sh)

In do_install_app():

```sh
#add symbolic link to mps for sms service
        ln -s /data/cozy-app/mps /data/cozy-storage/cozy.tools\:8080/.cozy_apps/mps
        if [ -d "data/cozy-app/mps/0.1.1" ]; then
                echo "data/cozy-app/mps/0.1.1 already here!";
        else
                mkdir /data/cozy-app/mps/0.1.1
                cp -a /data/cozy-app/mps/services/ /data/cozy-app/mps/0.1.1/
                echo "data/cozy-app/mps/0.1.1 just create!";
        fi
        # mkdir /data/cozy-app/mps/0.1.1
        # cp -a /data/cozy-app/mps/services/ /data/cozy-app/mps/0.1.1/
  echo "service added"
```

5/ Then, add konnector-node-run.sh in docker folder. This file is a script to store the service logs. It should contain:

```sh
#!/bin/bash

rundir="${1}"
cd $rundir
node index.js | tee /data/cozy-app/mps/services.log
```

Then, add this following row into the Dockerfile:

COPY konnector-node-run.sh /etc/cozy/konnector-node-run.sh

6/ Into the disableCSP.yml, you need to add one more configuration:

konnectors:
cmd: /etc/cozy/konnector-node-run.sh # run connectors with node

:::warning
You need to build and push the newest image on the forge registry when the step 6 is done.
You must have all files, into your docker folder, in LF mode of "End of Line Sequence" !
:::

Finally, you should rebuild the docker image of the cozy-stack:

```sh
cd docker
docker build .
docker tag <image_ID_created> registry.forge.grandlyon.com/web-et-numerique/factory/monparcourssocial/mps-usager/cozy-env-doctypes
docker push registry.forge.grandlyon.com/web-et-numerique/factory/monparcourssocial/mps-usager/cozy-env-doctypes:latest
```

7/ The path "/data/cozy-doctypes" is link to the volume of the cozy-app-dev image.

In order to mount this volume, you need to add it when you run the 'docker run' command :

```sh
   -v $PWD/cozy-doctypes:/data/cozy-doctypes
```

=> for this project :

```sh
docker run --rm -it -p 8080:8080 -p 5984:5984 -p 8025:8025 -v $PWD/build:/data/cozy-app/mps -v $PWD/data:/usr/local/couchdb/data -v $PWD/docker/disableCSP.yaml:/etc/cozy/cozy.yaml -v $PWD/cozy-doctypes:/data/cozy-doctypes registry.forge.grandlyon.com/web-et-numerique/factory/monparcourssocial/mps-usager/cozy-env-doctypes
```

## How to test your service ?

In the manifest.webapp, you need to declare the service:

```json
"services": {
  "smsProcess": {
    "type": "node",
    "file": "services/smsProcess/pilote.js"
  }
}
```

Then, in src\targets\services, you can add your service.

For this project, we have created a button in Home.component.tsx in order to generate trigger url to run the service:

```js
const getTrigger = async () => {
  const attrs = {
    type: '@webhook',
    worker: 'service',
    message: {
      name: 'smsProcess',
      slug: 'pilotedemo',
    },
  };

  const trigger = await client
    .getStackClient()
    .collection('io.cozy.triggers')
    .create(attrs);
  console.log('trigger', trigger);
};
```

It generates an trigger URL that we can call with our backend application :

> URL_COZY/job/webhook/trigger_id

URL_COZY represent the cozy-stack url of your environment and not your application URL.
